"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SecretsStorageImpl = void 0;
const config_1 = require("../../config");
const constants_1 = require("../../constants");
const logger_1 = require("../../logger");
const http_1 = require("../../utils/http");
const utils_1 = require("../../utils/utils");
const cache_1 = require("../cache");
const utils_2 = require("./utils");
class SecretsStorageImpl {
    secrets = [];
    logger;
    constructor() {
        this.logger = (0, logger_1.get)("SecretsStorageImpl");
    }
    async deleteAllSecrets() {
        await (0, cache_1.deleteFile)(constants_1.CACHE_SECRETS_FILE);
        this.secrets = [];
    }
    async recreateCalculatedValues() {
        for (let i = 0; i < this.secrets.length; i++) {
            this.secrets[i].id = i;
            if (!this.secrets[i].client_id) {
                this.logger.warn(`client id is not set for secret with ID ${i}, skipping update of secret information`);
                // eslint-disable-next-line no-continue
                continue;
            }
            this.secrets[i].customClient = (0, utils_2.isCustomClient)(this.secrets[i].client_id);
            // eslint-disable-next-line no-await-in-loop
            await this.updateUrls(i);
        }
    }
    async initializeStorage() {
        try {
            this.secrets = JSON.parse(await (0, cache_1.readFile)(constants_1.CACHE_SECRETS_FILE));
            if (!Array.isArray(this.secrets)) {
                this.logger.warn("secrets data is no array, initializing secrets to empty array");
                this.secrets = [];
            }
            await this.recreateCalculatedValues();
        }
        catch (err) {
            this.logger.warn("failed to read or parse secrets file", err);
        }
    }
    async getDefaultSecret() {
        return this.getSecretById(await this.getDefaultSecretId());
    }
    async getDefaultSecretId() {
        const tenantUrl = (0, utils_2.getTenantUrl)();
        const secret = this.secrets.find((s) => new URL(s.tenantUrl).host === new URL(tenantUrl).host);
        if (!secret) {
            throw new Error("Method not implemented.");
        }
        return secret.id;
    }
    ensureSecretsExist(id) {
        return !!this.secrets.at(id);
    }
    throwIfSecretsDoesntExist(id) {
        if (!this.ensureSecretsExist(id)) {
            throw new Error(`no secrets exist for id ${id}`);
        }
    }
    async getSecretById(id) {
        this.throwIfSecretsDoesntExist(id);
        return this.secrets.at(id);
    }
    removeSecretsFromArray(id) {
        this.secrets.splice(id, 1);
    }
    updateSecret(secret) {
        const tenantUrl = (0, utils_2.getTenantUrl)();
        let prevSecretIx = this.secrets.findIndex((s) => s.tenantUrl === tenantUrl);
        let newSecret;
        if (prevSecretIx > -1) {
            newSecret = { ...this.secrets[prevSecretIx], ...secret };
            this.secrets[prevSecretIx] = newSecret;
        }
        else {
            newSecret = {
                ...secret,
                id: this.secrets.length,
                tenantUrl,
                customClient: false,
            };
            this.secrets.push(newSecret);
            prevSecretIx = this.secrets.length - 1;
        }
        return prevSecretIx;
    }
    async updateUrls(secretIndex) {
        const secret = this.secrets[secretIndex];
        if (secret.client_id) {
            const config = (0, config_1.get)();
            let oauth = {};
            if (!secret.customClient &&
                (!secret.authorization_url || !secret.token_url)) {
                oauth = (await (0, http_1.fetch)({
                    method: "GET",
                    url: `${config.tenantUrl}/oauth`,
                })).data;
            }
            let authorizationUrl = secret.authorization_url;
            if (!authorizationUrl) {
                if (!secret.customClient) {
                    authorizationUrl = oauth.authorizationUrl;
                }
                else {
                    authorizationUrl = config.authorizationUrl;
                }
            }
            let tokenUrl = secret.token_url;
            if (!tokenUrl) {
                if (!secret.customClient) {
                    tokenUrl = oauth.tokenUrl;
                }
                else {
                    tokenUrl = config.tokenUrl;
                }
            }
            if (!tokenUrl || !authorizationUrl) {
                throw new Error("invalid token url or authorization url");
            }
            this.secrets[secretIndex].authorization_url =
                (0, utils_1.removeQueryParametersFromUrl)(authorizationUrl);
            this.secrets[secretIndex].token_url = (0, utils_1.removeQueryParametersFromUrl)(tokenUrl);
        }
    }
    async storeSecret(secret) {
        this.updateSecret(secret);
        await this.recreateCalculatedValues();
    }
    deleteSecretById(id) {
        this.throwIfSecretsDoesntExist(id);
        this.removeSecretsFromArray(id);
    }
    async synchronizeSecretsToStorage() {
        try {
            await (0, cache_1.writeFile)(constants_1.CACHE_SECRETS_FILE, JSON.stringify(this.secrets));
        }
        catch (err) {
            this.logger.error("failed to synchronize secrets to storage", err);
        }
    }
    getAllSecrets() {
        return this.secrets;
    }
}
exports.SecretsStorageImpl = SecretsStorageImpl;
