"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCode = exports.retrieveCode = exports.refreshToken = void 0;
const http_1 = require("http");
const fs_extra_1 = require("fs-extra");
const logger_1 = require("../../../../../logger");
const utils_1 = require("../utils");
const setAuthorization_1 = require("./setAuthorization");
const constants_1 = require("../../../../../constants");
const options_1 = require("../../../../../utils/options");
const utils_2 = require("../../../../../utils/utils");
const SecretsStorageSingleton_1 = require("../../../../../cache/secrets/SecretsStorageSingleton");
const getLogger = () => (0, logger_1.get)("commands.handler.authentication.oauth.tokenProvider.utils.refreshToken");
const refreshToken = async (forceRefresh = false) => {
    const { info: logInfo, debug } = getLogger();
    const secrets = await SecretsStorageSingleton_1.SecretsStorageSingleton.SINGLETON.getDefaultSecret();
    if ((!forceRefresh && !secrets.expires_after) || !secrets.refresh_token) {
        throw new Error("invalid secrets information");
    }
    logInfo("checking token expiry date");
    if (forceRefresh || (0, utils_1.isExpired)(secrets.expires_after)) {
        debug("access token is expired, refreshing token");
        await (0, utils_1.readToken)({
            refresh_token: secrets.refresh_token,
            grant_type: "refresh_token",
        });
        await (0, setAuthorization_1.updateAuthorization)();
    }
    else {
        debug("access token is not expired");
    }
};
exports.refreshToken = refreshToken;
const retrieveCode = async () => 
// eslint-disable-next-line no-async-promise-executor
new Promise(async (resolve, reject) => {
    const { debug, error } = getLogger();
    try {
        const secrets = await SecretsStorageSingleton_1.SecretsStorageSingleton.SINGLETON.getDefaultSecret();
        const defaultPort = secrets.customClient ? "8080" : "65000";
        const PORT = parseInt(process.env.CLI_HTTP_PORT || defaultPort, 10);
        let timeout;
        const server = (0, http_1.createServer)(async (req, res) => {
            clearTimeout(timeout);
            const code = new URL(req.url
                ? `http://localhost:${PORT}${req.url}`
                : "http://no-code.damn").searchParams.get("code");
            let file;
            try {
                file = await (0, fs_extra_1.readFile)(code ? constants_1.PATH_TO_SUCCESS_HTML : constants_1.PATH_TO_ERROR_HTML, "utf8");
            }
            catch (err) {
                debug("failed to read file for code", code, err);
                file =
                    "<html><body>Ops, something went wrong! Please try again.</body></html>";
            }
            res.writeHead(200, {
                "Content-Type": "text/html",
                "Content-Security-Policy": "frame-ancestors 'none'",
                "X-Frame-Options": "DENY",
            });
            res.end(file, "utf8");
            server.close();
            if (code) {
                resolve(code);
            }
            else {
                reject(new Error("no code found in callback URI"));
            }
        });
        timeout = setTimeout(() => {
            server.close();
            reject(new Error("Did not receive a code within 30 seconds"));
        }, 30 * 1000);
        server.listen(PORT);
        debug(`started http server at localhost:${PORT}`);
    }
    catch (err) {
        error("failed to instantiate server", err);
        reject(new Error("failed to instantiate server", { cause: err }));
    }
});
exports.retrieveCode = retrieveCode;
const getCode = async (authorizeUrl, clientId) => {
    const { debug } = getLogger();
    try {
        return (0, options_1.getOptionValueFromConfig)(constants_1.OPTION_CODE);
    }
    catch (err) {
        debug("failed to retrieve code from options", err);
        return (await Promise.all([
            (0, exports.retrieveCode)(),
            (0, utils_2.openUrlInBrowser)(authorizeUrl, {
                response_type: "code",
                client_id: clientId,
            }),
        ]))[0];
    }
};
exports.getCode = getCode;
